#!/usr/bin/python3
# This python script is used to clean the text (from the BGA abstracts) and
# categorize the abstracts for Becker et al. (2021).

import pandas as pd
import argparse


def categorize(df, keyword, col_tomatch, col_tomark, marker):
	df.loc[df[col_tomatch].str.contains(pat = keyword), col_tomark] = marker
	return df

def process_arguments():
    """
    Parses command line arguments.

    Args:
    -----
    None

    Returns:
    --------
    parser: :class: argparse.Namespace
        arguments passed in from command line
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('--pathin', required=True, type=str)
    parser.add_argument('--pathout', required=True, type=str)
    parser.add_argument('--firstpage', required=True, type=int)
    parser.add_argument('--lastpage', required=True, type=int)
    parser.add_argument('--bga_string', required=True, type=str)
    parser.add_argument('--twin_registries', nargs='*', type=str,
    	default=['Michigan State University Twin Registry',
    	'Osaka University Center for Twin Research', 'Boston University Twin Project'])
    return parser.parse_args()


if __name__ == "__main__":
	# read args
	args = process_arguments()

	# read file
	file = open(args.pathin,"r")
	x = file.readlines()

	# delete publishing stuff
	x1 = list(filter(lambda a: a != 'Q Springer\n', x))
	x1 = [sub.replace('\x0c', '') for sub in x1]
	x1 = [sub.replace(args.bga_string, '') for sub in x1]

	# delete page numbers
	for i in range(args.firstpage, args.lastpage):
		x1 = list(filter(lambda a: a != f'{i}\n', x1))

	# Combine lines in the same paragraph
	x2 = x1
	index = 0
	while index+2 <= len(x2):
		if x2[index] == '\n':
			index+=1
			continue
		if x2[index] != '\n' and x2[index+1] == '\n':
			index+=2
			continue
		if x2[index] != '\n' and x2[index+1] != '\n':
			# concatenate
			x2[index] = x2[index] + x2[index+1]
			# remove the thing that was concatenated
			del x2[index+1:index+2]

	# delete empty lines
	x3 = list(filter(lambda a: a != '\n', x2))
	x4 = list(filter(lambda a: a != '\x0c', x3))

	# replace \n with a space
	x5 = [sub.replace('\n', ' ') for sub in x4]

	# remove leading and trailing spaces
	x6 = [c.strip() for c in x5]
	# remove empty lines
	x6 = list(filter(lambda a: a != '', x6))

	# change university twin registries to lowercase
	for word in args.twin_registries:
		x6 = [sub.replace(word, word.lower()) for sub in x6]

	# concatenate all the authors together
	index = 0
	while index+2 <= len(x6):
		if ('University' in x6[index] or 'College' in x6[index] or 'Universiteit' in x6[index] \
			or 'Department' in x6[index] or 'Institute' in x6[index] or 'Institutet' in x6[index]) and \
			('University' in x6[index+1] or 'College' in x6[index+1] or 'Universiteit' in x6[index+1] \
			or 'Department' in x6[index+1] or 'Institute' in x6[index+1] or 'Institutet' in x6[index+1]):
			# concatenate
			x6[index] = x6[index] + x6[index+1]
			# remove the thing that was concatenated
			del x6[index+1:index+2]
		else:
			index+=1
			continue

	# concatenate the abstracts
	index = 0
	while index+2 < len(x6):
		if ('University' in x6[index] or 'College' in x6[index] or 'Universiteit' in x6[index] \
			or 'Department' in x6[index] or 'Institute' in x6[index] or 'Institutet' in x6[index]) or \
			('University' in x6[index+1] or 'College' in x6[index+1] or 'Universiteit' in x6[index+1] \
			or 'Department' in x6[index+1] or 'Institute' in x6[index+1] or 'Institutet' in x6[index+1]) or \
			('University' in x6[index+2] or 'College' in x6[index+2] or 'Universiteit' in x6[index+2] \
			or 'Department' in x6[index+2] or 'Institute' in x6[index+2] or 'Institutet' in x6[index+2]):
			# move on
			index+=1
			continue
		else:
			# concatenate
			x6[index] = x6[index] + x6[index+1]
			# remove the thing that was concatenated
			del x6[index+1:index+2]

	# remove hyphens between lines
	x6 = [sub.replace('- ', '') for sub in x6]

	# organize data
	papers = []
	while len(x6) > 0:
		if len(x6) >= 3:
			single = [x6[0], x6[1], x6[2]]
			papers.append(single)
			del x6[0:3]
		else:
			papers.append(x6)
			x6 = []

	# put in df
	df = pd.DataFrame(papers, columns=['Title', 'Author', 'Abstract'])
	df.drop_duplicates(inplace=True)


	################################# categorize papers ###########################################
	df_lower = df.copy()
	for col in list(df_lower.columns):
		df_lower[col] = df_lower[col].str.lower()

	"""
	Categorize PGS studies
	See Methods Section VI for full description of algorithm
	"""
	# categorize: pgs
	pgs_keywords = ['pgs', 'prs', 'pgrs', 'polygenic score', 'polygenic risk score', 'genetic risk score', 'grs']
	df_lower['pgs'] = 0
	for word in pgs_keywords:
		df_lower = categorize(df=df_lower, keyword=word, col_tomatch='Title', col_tomark='pgs', marker=1)
		df_lower = categorize(df=df_lower, keyword=word, col_tomatch='Abstract', col_tomark='pgs', marker=1)


	"""
	Categorize candidate gene studies
	See Methods Section VI for full description of algorithm
	"""
	df_lower['candidate gene'] = 0

	# match title
	candidate_keywords = ['candidate gene', 'htr2', 'maoa', '5-htt', '5htt', 'drd', 'slc6', 'bdnf', 'comt', 'tph',
	'mthfr', 'apoe', 'dtnbp1', 'dbh', 'abcb1', 'vntr', 'crhr', 'akt', 'nrg', 'avp',
	'rs1', 'rs2', 'rs3', 'rs4', 'rs5', 'rs6', 'rs7', 'rs8', 'rs9', 'rs0']
	for word in candidate_keywords:
		df_lower = categorize(df=df_lower, keyword=word, col_tomatch='Title', col_tomark='candidate gene', marker=1)

	# match abstract: 'candidate'
	df_lower['count1'] = df_lower['Abstract'].str.count('candidate')
	df_lower['count2'] = df_lower['Abstract'].str.count('candidate gene')
	df_lower.loc[(df_lower['count1'] >= 2) & (df_lower['count2'] >= 1), 'candidate gene'] = 1
	df_lower.drop(columns=['count1', 'count2'], inplace=True)

	# match abstract: gene names
	candidate_keywords.remove('candidate gene')
	for word in candidate_keywords:
		df_lower = categorize(df=df_lower, keyword=word, col_tomatch='Abstract', col_tomark='candidate gene', marker=1)

	# remove if wide association analysis is in abstract
	gwas_keywords = ['wide association analysis', 'wide association study', 'gwas']
	for word in gwas_keywords:
		df_lower = categorize(df=df_lower, keyword=word, col_tomatch='Abstract', col_tomark='candidate gene', marker=0)
		df_lower = categorize(df=df_lower, keyword=word, col_tomatch='Title', col_tomark='candidate gene', marker=0)




	"""
	Categorize twin studies
	See Methods Section VI for full description of algorithm
	"""
	df_lower['twin study'] = 0
	df_lower['big_abstract'] = df['Abstract'].copy()
	df_lower['big_title'] = df['Title'].copy()

	# mark 1 if abstract mentions 'twin' at least twice
	df_lower['count'] = df_lower['Abstract'].str.count('twin')
	df_lower.loc[df_lower['count'] >= 2, 'twin study'] = 1
	df_lower.drop(columns=['count'], inplace=True)

	# mark 1 if title contains 'twin'
	df_lower = categorize(df=df_lower, keyword='twin', col_tomatch='Title', col_tomark='twin study', marker=1)

	# mark 1 if abstract contains keywords
	twin_keywords = ['twin registry', 'center for twin research', 'twin project', 'twin panel', 'twin study at the',
	'twin study (lts)', '(rfab) twin study', 'twin register', 'twin pairs', 'nonidentical twins', 'identical twins',
	'pairs of twins', 'twin sample', ' mz', ' dz', 'monozygotic', 'dizygotic', 'pairs of twins',
	'adopted', 'adoptee', 'adoptive', 'adoption design', 'biological parent',
	'adoptive parent', 'adoption-sibling', 'genetically-unrelated', 'genetically-related', 'siblings reared together',
	'siblings reared apart', 'mother and child', 'father and child', 'parent and child', 'intergenerational', 'transracial',
	'biometric', 'path analy', 'cholesky', 'children-of-twins', 'children of twins', 'common environment', 'unique environment']
	big_keywords = ['ACE', 'ACDE']

	# mark 1 if title/abstract contains twin keyword
	for word in twin_keywords:
		df_lower = categorize(df=df_lower, keyword=word, col_tomatch='Abstract', col_tomark='twin study', marker=1)
	for word in twin_keywords:
		df_lower = categorize(df=df_lower, keyword=word, col_tomatch='Title', col_tomark='twin study', marker=1)
	# mark if title/abstract contains big keyword
	for word in big_keywords:
		df_lower = categorize(df=df_lower, keyword=word, col_tomatch='big_title', col_tomark='twin study', marker=1)
		df_lower = categorize(df=df_lower, keyword=word, col_tomatch='big_abstract', col_tomark='twin study', marker=1)

	# drop irrelevant cols
	df_lower.drop(columns=['big_abstract', 'big_title'], inplace=True)

	################################ format data ################################
	df['Uses PGS'] = df_lower['pgs'].copy()
	df['Twin Study'] = df_lower['twin study'].copy()
	df['Candidate Gene Study'] = df_lower['candidate gene'].copy()

	################################# export ###########################################
	df.to_csv(args.pathout, header=True, index=False, sep=',')
